#include "chat_dialog.h"
#include "ui_chat_dialog.h"
#include "followupslistmodel.h"
#include <QMessageBox>
#include "mso_inifile.h"
#include "mainwindow.h"

using namespace xmlrpc;

ChatDialog::ChatDialog(FollowupsListModel *followupsModel, const QString & sessionKey, MSO_IniFile *settingsObj, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ChatDialog)
{
    this->sessionKey = sessionKey;
    this->settingsObj = settingsObj;

    ui->setupUi(this);

    this->model = followupsModel;
    ui->tableView->setModel(followupsModel);

    ui->tableView->hideColumn(0);
    ui->tableView->hideColumn(2);

    ui->tableView->horizontalHeader()->setResizeMode(1, QHeaderView::ResizeToContents);
    ui->tableView->horizontalHeader()->setResizeMode(3, QHeaderView::ResizeToContents);
    ui->tableView->horizontalHeader()->setResizeMode(4, QHeaderView::Stretch);
    ui->tableView->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);

    this->xmlRpc_GetTicketFollowups(followupsModel->ticketId());
    ui->lineEdit->setFocus();
}

ChatDialog::~ChatDialog()
{
    delete ui;
}

void ChatDialog::xmlRpc_GetTicketFollowups(int ticketId)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = this->sessionKey;
    Variant id = ticketId;
    client->request("xmlrpc.getTicketFollowups", sessionKey, id);
}

void ChatDialog::xmlRpc_NewTicketFollowup(int ticketId, const QString & text)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = this->sessionKey;
    Variant id = ticketId;
    Variant content = text;
    client->request("xmlrpc.newTicketFollowup", sessionKey, id, content);
}

xmlrpc::Client *ChatDialog::prepareXmlRpcClient()
{
    xmlrpc::Client *client = new Client(this);

    QObject::connect(client,SIGNAL(done(int, QVariant)),
                            this, SLOT(onDone(int, QVariant)));
    QObject::connect(client,SIGNAL(failed( int, int, QString )),
                 this, SLOT(processFault( int, int, QString )) );

    QUrl url(this->settingsObj->helpdeskServerUrl);
    client->setHost(url.host(), 80, url.path());
    client->setUserAgent("sphere-client");

    return client;
}

void ChatDialog::processFault( int requestId, int errorCode, QString errorString )
{
    Q_UNUSED( requestId );

    QMessageBox::warning(this, tr("Ошибка запроса"),
            tr("Ошибка XML-RPC запроса.\n\nКод ошибки: %1\n%2\n") \
            .arg(errorCode).arg(errorString),
            QMessageBox::Ok );

    QCoreApplication::exit(-1);
}

void ChatDialog::onDone(int, QVariant variant)
{
    // QVariant - карта со значениями
    QMultiMap<QString , QVariant> map(variant.toMap());
    QString flerror = map.value("flerror").toString(); // код ошибки
    // сообщение (объяснение ошибки, если она была)
    QString message = map.value("message").toString();

    if(flerror.size() != 0)
    {
        QMessageBox::warning(0,
                             flerror,
                             message,
                             QMessageBox::Ok,Qt::NoButton);

        QCoreApplication::exit(-1);
        return;
    }

    QString method = map.value("xmlrpc_response.method").toString();

    if(method == QString("getTicketFollowups"))
    {
        if(map.value("xmlrpc_response.result").toInt() == 1)
        {
            QList<QVariant> rows = map.value("xmlrpc_response.rows").toList();
            this->model->setupModel(rows);

            ui->tableView->horizontalHeader()->setResizeMode(1, QHeaderView::ResizeToContents);
            ui->tableView->horizontalHeader()->setResizeMode(3, QHeaderView::ResizeToContents);
            ui->tableView->horizontalHeader()->setResizeMode(4, QHeaderView::Stretch);
            ui->tableView->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);

            ui->tableView->scrollToBottom();
        }
        else
        {
            QMessageBox::warning(0,
                                     tr("Ошибка!"),
                                     map.value("xmlrpc_response.resultMessage").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            QCoreApplication::exit(-1);
        }
        return;
    }

    if(method == QString("newTicketFollowup"))
    {
        if(map.value("xmlrpc_response.result").toInt() == 1)
        {
            emit updateFollowupsCount();
            this->model->addFollowup(map.value("xmlrpc_response.followupDate"), map.value("xmlrpc_response.followupContent").toString());
            ui->tableView->horizontalHeader()->setResizeMode(1, QHeaderView::ResizeToContents);
            if(this->model->rowCount() == 1)
            {
                ui->tableView->horizontalHeader()->setResizeMode(3, QHeaderView::ResizeToContents);
                ui->tableView->horizontalHeader()->setResizeMode(4, QHeaderView::Stretch);
                ui->tableView->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
            }
            ui->tableView->scrollToBottom();
        }
        else
        {
            QMessageBox::warning(0,
                                     tr("Ошибка!"),
                                     map.value("xmlrpc_response.resultMessage").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            QCoreApplication::exit(-1);
        }
        return;
    }

    QMessageBox::warning(0,
                             tr("Ошибка!"),
                             tr("Ответ сервера XML-RPC не распознан!"),
                             QMessageBox::Ok,Qt::NoButton);
    QCoreApplication::exit(-1);
}

void ChatDialog::on_pushButton_clicked()
{
    this->xmlRpc_NewTicketFollowup(this->model->ticketId(), ui->lineEdit->text());
    ui->lineEdit->clear();
    ui->lineEdit->setFocus();
}
